#pragma once
#include <Windows.h>

#define GLREDRAW        WM_USER+1

class GLRenderer
{
public:
  GLRenderer();
  ~GLRenderer();

  void Init(HWND parent, HINSTANCE hinstance, int x, int y, int width, int height);
  void OnSize(UINT nType, int cx, int cy);
  void SetWindowPos(int x, int y, int width, int height);
  void GetWindowPos(int& x, int& y, int& width, int&height);
  void Draw();
  bool HitTest(int x, int y);
  void ResetCamera(bool redraw);
  bool Created();

  void OnLButtonDown(UINT nType, int x, int y);
  void OnMouseMove(UINT nType, int x, int y);
  void OnLButtonUp(UINT nType, int x, int y);
  void OnMouseWheel(WPARAM wParam, int x, int y);

  void SetObjectInfo(const int noofvers, float* verts, const int noofpolys, int* indices);

  HWND parent, hwnd;

private:
  HINSTANCE hinstance;
  int x, y, width, height;
  bool created;
  
  float rotation[3];
  float eye[3], centre[3], up[3];
      
  bool mousedown;
  POINT lastpoint;

  int noofverts, noofpolys;
  float* verts;
  int* indices;

  void SetupLighting();
  void CalcRotationMatrix(float mat[16]);
  void CalcLookAt(bool fromeye);

  void InitAux();

  friend LRESULT CALLBACK GLWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
  friend DWORD WINAPI RendererThread(LPVOID lpParameter);
};

inline bool GLRenderer::HitTest(int x, int y)
{
  return (x>=this->x && x<=(this->x+width) && y>=this->y && y<=(this->y+height));
}

inline bool GLRenderer::Created()
{
  return created;
}