#include "MainWnd.h"
#include <assert.h>

MainWnd* mainwnd;
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

MainWnd::MainWnd(HINSTANCE hInstance, LPSTR lpCmdLine) : hinstance(hInstance)
{
	WNDCLASSEX classname;
  classname.cbSize=sizeof(WNDCLASSEX); 
  classname.style=CS_HREDRAW|CS_VREDRAW;
  classname.lpfnWndProc=(WNDPROC) WndProc;
  classname.cbClsExtra=0;
  classname.cbWndExtra=0;
  classname.hInstance=hInstance;
  classname.hIcon=LoadIcon(NULL, IDI_WINLOGO);
  classname.hCursor=LoadCursor(NULL, IDC_ARROW);
  classname.hbrBackground=::GetSysColorBrush(COLOR_3DFACE);
  classname.lpszMenuName=NULL;
  classname.lpszClassName=L"PCL_PCtoPoly";
  classname.hIconSm=LoadIcon(NULL, IDI_WINLOGO);
  RegisterClassEx(&classname);

  int width=800;
  int height=640;
  int offx=(::GetSystemMetrics(SM_CXSCREEN)-width)/2;
  int offy=(::GetSystemMetrics(SM_CYSCREEN)-height)/2;

  filename[0]='\0';
  if (lpCmdLine && lpCmdLine[0])
  {
    if (lpCmdLine[0]=='\"')
      strcpy_s(filename, MAX_PATH, lpCmdLine+1);
    else
      strcpy_s(filename, MAX_PATH, lpCmdLine);

    int len=(int) strlen(filename);
    if (filename[len-1]=='\"')
      filename[len-1]='\0';
  }
  if (!filename[0])
    strcpy_s(filename, MAX_PATH, "PointsToPolys.dat");

  ::mainwnd=this;
  hwnd=CreateWindowEx(NULL, L"PCL_PCtoPoly", L"Kinecting up the Past - PCL_PCtoPoly", WS_OVERLAPPEDWINDOW|WS_VISIBLE, offx, offy, width, height, NULL, NULL, hInstance, NULL);

  MSG msg;
  do
  {
    if (GetMessage(&msg, NULL, 0, 0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  } while (msg.message!=WM_QUIT);
}

MainWnd::~MainWnd()
{

}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message) 
  {
  case WM_CREATE:
    mainwnd->Init(hWnd);
    return 0;
    break;
  case WM_SIZE:
    mainwnd->OnSize((DWORD) wParam, (UINT) (lParam&0xFFFF), (UINT) (lParam>>16));
    break;
  case WM_PAINT:
    {
      PAINTSTRUCT paint;
      BeginPaint(hWnd, &paint);
      mainwnd->OnDraw();
      EndPaint(hWnd, &paint);
    }
    break;
  case WM_DESTROY:
    PostQuitMessage(0);
    break;
  case WM_CLOSE:
    DestroyWindow(hWnd);
    PostQuitMessage(0);
    break;
  case WM_LBUTTONDOWN:
    mainwnd->OnLButtonDown(wParam, lParam&0xFFFF, lParam>>16);
    break;
  case WM_MOUSEMOVE:
    mainwnd->OnMouseMove(wParam, lParam&0xFFFF, lParam>>16);
    break;
  case WM_LBUTTONUP:
    mainwnd->OnLButtonUp(wParam, lParam&0xFFFF, lParam>>16);
    break;
  case WM_MOUSEWHEEL:
    mainwnd->OnMouseWheel(wParam, lParam&0xFFFF, lParam>>16);
    return 0;
    break;
  case WM_KEYDOWN:
    mainwnd->OnKeyDown(wParam, lParam&0xFFFF, lParam>16);
    break;
  case WM_COMMAND:
    {
/*      WORD wmID=(WORD) (wParam&0xFFFF);
      WORD wmevent=(WORD) (wParam>>16);
      if (wmevent==CBN_SELCHANGE)
        mainwnd->OnComboChange();*/
    }
    break;
/*  case WM_KEYDOWN:
    mainwnd->OnKeyDown(wParam, lParam&0xFFFF, lParam>>16);
    break;
  case WM_KEYUP:
    mainwnd->OnKeyUp(wParam, lParam&0xFFFF, lParam>>16);
    break;*/
  case GLREDRAW:
    mainwnd->OnGLRedraw(lParam);
    break;
  default:
    return DefWindowProc(hWnd, message, wParam, lParam);
  }
  return DefWindowProc(hWnd, message, wParam, lParam);
}

void MainWnd::Init(HWND hwnd)
{
  this->hwnd=hwnd;

  renderer.Init(hwnd, hinstance, 0, 410, 240, 180);

  while (!renderer.Created())
    Sleep(50);

  FILE* file=NULL;
  fopen_s(&file, filename, "rb");
  if (file)
  {
    int noofpts;
    fread(&noofpts, 4, 1, file);

    int offset=0;
    float* xyzs=(float*) malloc(4*3*noofpts);
    float* nxyzs=(float*) malloc(4*3*noofpts);
    for (int i=0; i<noofpts; i++)
    {
      fread(xyzs+offset, 4, 3, file);
      fread(nxyzs+offset, 4, 3, file);
      offset+=3;
    }
    fclose(file);
    
    OBJECT* obj=PCConvert(noofpts, xyzs, nxyzs, filename);
    renderer.SetObjectInfo(obj->noofvers, obj->verts, obj->noofpolys, obj->indices);
    
    free(obj);

    free(xyzs);
    free(nxyzs);
  }
  else
    MessageBox(hwnd, L"Cannot open file", L"PCL_PCtoPoly", MB_ICONERROR);
}

void MainWnd::OnSize(UINT nType, int cx, int cy)
{
  renderer.SetWindowPos(0, 0, cx, cy);
}

void MainWnd::OnLButtonDown(UINT nFlags, int x, int y)
{
}

void MainWnd::OnMouseMove(UINT nFlags, int x, int y)
{
}

void MainWnd::OnLButtonUp(UINT nFlags, int x, int y)
{
}

void MainWnd::OnMouseWheel(WPARAM wParam, int x, int y)
{
  RECT rect;
  ::GetWindowRect(hwnd, &rect);
  x-=rect.left+GetSystemMetrics(SM_CXSIZEFRAME);
  y-=rect.top+GetSystemMetrics(SM_CYSIZEFRAME)+GetSystemMetrics(SM_CYCAPTION);
  if (renderer.HitTest(x, y))
    renderer.OnMouseWheel(wParam, x, y);
}

void MainWnd::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
  if (nChar=='R')
    renderer.ResetCamera(true);
}

void MainWnd::OnDraw()
{
  OnGLRedraw(3);
}

void MainWnd::OnGLRedraw(LPARAM lParam)
{
  SendMessage(renderer.hwnd, GLREDRAW, 0, 0);
}