#include "PCConvert.h"
#include <pcl/point_types.h>
#include <pcl/kdtree/kdtree_flann.h>
//#include <pcl/octree/octree.h>
#include <pcl/features/normal_3d.h>
#include <pcl/surface/gp3.h>
#include <Windows.h>
#include <stdio.h>
#include <assert.h>

int FormatNumber(wchar_t* buffer, const int bufflen, double value, int dps)
{
  int pos=0;
  if (value<0)
  {
    buffer[0]='-';
    value=-value;
    pos=1;
  }

  int tval=(int) value;
  _itow_s(tval, buffer+pos, bufflen-pos, 10);
  pos+=(int) wcslen(buffer+pos);

  if (dps>0)
  {
    buffer[pos++]='.';
    while (dps>0)
    {
      value=(value-tval)*10;
      tval=(int) value;
      _itow_s(tval, buffer+pos, bufflen-pos, 10);
      pos+=(int) wcslen(buffer+pos);
      --dps;
    }
  }
  buffer[pos]='\0';
  return pos;
}

bool Test()
{
  pcl::PointCloud<pcl::PointXYZ>::Ptr vertices(new pcl::PointCloud<pcl::PointXYZ>);

  int noofpts=5;
/*  noofpts=1;
  vertices->width=noofpts;
  vertices->height=1;
  vertices->is_dense=true;
  vertices->points.resize(noofpts);*/
  vertices->points.resize(noofpts);

  int i, j, pos=0;
  for (i=0; i<noofpts; i++)
  {
    int offset=3*i;
    vertices->points[i].x=i;
    vertices->points[i].y=2*i;
    vertices->points[i].z=3*i;
  }

  pcl::search::KdTree<pcl::PointXYZ>* tree2=new pcl::search::KdTree<pcl::PointXYZ>;
  tree2->setInputCloud(vertices);

  return true;
}

OBJECT* PCConvert(int noofpts, const float* xyzs, const float* nxyzs, const char* filename)
{
  OBJECT* res=(OBJECT*) malloc(sizeof(OBJECT));
  memset(res, 0, sizeof(OBJECT));

//  if (Test())
//    return res;

  pcl::PointCloud<pcl::PointNormal>::Ptr vertices(new pcl::PointCloud<pcl::PointNormal>);

//  noofpts=852;
/*  noofpts=1;
  vertices->width=noofpts;
  vertices->height=1;
  vertices->is_dense=true;
  vertices->points.resize(noofpts);*/
  vertices->points.resize(noofpts);

  int i, j, pos=0;
  for (i=0; i<noofpts; i++)
  {
    int offset=3*i;
    vertices->points[pos].x=xyzs[offset];
    vertices->points[pos].y=xyzs[offset+1];
    vertices->points[pos].z=xyzs[offset+2];

    vertices->points[pos].normal_x=nxyzs[offset];
    vertices->points[pos].normal_y=nxyzs[offset+1];
    vertices->points[pos].normal_z=nxyzs[offset+2];

    vertices->points[pos].curvature=0;

    ++pos;
  }

//  pcl::PointNormal temp=vertices->points[pos-1];

  pcl::search::KdTree<pcl::PointNormal>::Ptr tree2(new pcl::search::KdTree<pcl::PointNormal>);
  //tree2->setInputCloud(vertices);
  pcl::PolygonMesh triangles;

  pcl::GreedyProjectionTriangulation<pcl::PointNormal> gp3;
  gp3.setSearchRadius(0.1);
  gp3.setMu(2.5);
  gp3.setMaximumNearestNeighbors(100);
  gp3.setMaximumSurfaceAngle(M_PI/4); // 45 degrees
  gp3.setMinimumAngle(M_PI/18); // 10 degrees
  gp3.setMaximumAngle(2*M_PI/3); // 120 degrees
  gp3.setNormalConsistency(false);

  MessageBox(NULL, L"Starting reconstruction", L"PCL_PCtoPoly", MB_ICONINFORMATION);

  gp3.setSearchMethod(tree2);
  gp3.setInputCloud(vertices);
  gp3.reconstruct(triangles);

  res->noofvers=noofpts;
  res->verts=(float*) malloc(4*3*res->noofvers);
  memcpy(res->verts, xyzs, 4*3*res->noofvers);

  res->noofpolys=triangles.polygons.size();
  res->indices=(int*) malloc(4*3*res->noofpolys);
  for (i=0; i<res->noofpolys; i++)
  {
    res->indices[3*i]=triangles.polygons[i].vertices[0];
    res->indices[3*i+1]=triangles.polygons[i].vertices[1];
    res->indices[3*i+2]=triangles.polygons[i].vertices[2];
  }

  char cbuff[100];
  wchar_t wbuff[MAX_PATH];

  MultiByteToWideChar(CP_ACP, 0, filename, -1, wbuff, MAX_PATH);
  ::DeleteFile(wbuff);

  FILE* file=NULL;
  char path[MAX_PATH];
  strcpy_s(path, MAX_PATH, filename);
  char* ext=strrchr(path, '.');
  if (ext)
    ext[0]='\0';
  strcat_s(path, MAX_PATH, ".wrl");
  fopen_s(&file, path, "wb");

  fwrite("#VRML V1.0 ascii\r\n\r\n", 1, 20, file);
  fwrite("Separator\r\n{\r\n", 1, 14, file);
  fwrite("  Coordinate3\r\n  {\r\n    point\r\n    [", 1, 36, file);

  int count=0;
  for (i=0; i<res->noofvers; i++)
  {
    if (count++)
      fwrite(",", 1, 1, file);
        
    fwrite("\r\n      ", 1, 8, file);

    for (j=0; j<3; j++)
    {
      FormatNumber(wbuff, 100, res->verts[3*i+j], 5);
      WideCharToMultiByte(CP_ACP, 0, wbuff, -1, cbuff, 100, 0, NULL);
      fwrite(cbuff, 1, strlen(cbuff), file);
      if (j!=2)
        fwrite(" ", 1, 1, file);
    }
  }
  fwrite("\r\n    ]\r\n  }", 1, 12, file);

  fwrite("\r\nIndexedFaceSet { coordIndex [\r\n", 1, 33, file);
  count=0;
  for (i=0; i<res->noofpolys; i++)
  {
    if (count++)
      fwrite(",\r\n", 1, 3, file);

    for (j=0; j<3; j++)
    {
      assert(res->indices[3*i+j]<res->noofvers);
      _itoa_s(res->indices[3*i+j], cbuff, 100, 10);
      fwrite(cbuff, 1, strlen(cbuff), file);
      fwrite(", ", 1, 2, file);
    }
    fwrite("-1", 1, 2, file);
  }
  fwrite("]}", 1, 2, file);
  fwrite("\r\n}", 1, 3, file);

  fclose(file);

  MessageBox(NULL, L"File Saved", L"PCL_PCtoPoly", MB_ICONINFORMATION);

//  std::vector<int> parts = gp3.getPartIDs();
//  std::vector<int> states = gp3.getPointStates();


  return res;
}